##############################################################################
# 
# Zope Public License (ZPL) Version 0.9.7
# ---------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

"""Folder object

Folders are the basic container objects and are analogous to directories.

$Id: Folder.py,v 1.59 1999/01/29 15:41:39 brian Exp $"""

__version__='$Revision: 1.59 $'[11:-2]

import Globals, SimpleItem, Acquisition, mimetypes, content_types
from Globals import HTMLFile
from ObjectManager import ObjectManager
from PropertyManager import PropertyManager
from AccessControl.Role import RoleManager
from CopySupport import CopyContainer
from FindSupport import FindSupport
from Image import Image, File
from string import rfind, lower


manage_addFolderForm=HTMLFile('folderAdd', globals())

def manage_addFolder(self,id,title='',createPublic=0,createUserF=0,
                         REQUEST=None):
    """Add a new Folder object with id *id*.

    If the 'createPublic' and 'createUserF' parameters are set to any true
    value, an 'index_html' and a 'UserFolder' objects are created respectively
    in the new folder.
    """
    i=self.folderClass()()
    i.id=id
    i.title=title
    self._setObject(id,i)
    if createUserF:  i.manage_addUserFolder()
    if createPublic: i.manage_addDTMLDocument(id='index_html',title='')
    if REQUEST is not None: return self.manage_main(self,REQUEST,update_menu=1)

class Folder(ObjectManager, PropertyManager, RoleManager, SimpleItem.Item,
             CopyContainer, FindSupport):
    """
    The basic container object in Principia.  Folders can hold almost all
    other Principia objects.
    """
    meta_type='Folder'
    id       ='folder'
    title    ='Folder object'
    icon     ='p_/folder'

    _properties=({'id':'title', 'type': 'string'},)

    meta_types=()
    dynamic_meta_types=(
        # UserFolderHandler.meta_types_
        )

    manage_options=(
    {'label':'Contents', 'action':'manage_main',
     'target':'manage_main'},
    {'label':'Properties', 'action':'manage_propertiesForm',
     'target':'manage_main'},
    {'label':'Security', 'action':'manage_access',
     'target':'manage_main'},
    {'label':'Undo', 'action':'manage_UndoForm',
     'target':'manage_main'},
    {'label':'Find', 'action':'manage_findFrame',
     'target':'manage_main'},
    )

    __ac_permissions__=(
        ('View', ()),
        ('View management screens',
         ('manage','manage_menu','manage_main','manage_copyright',
          'manage_tabs','manage_propertiesForm','manage_UndoForm',
          'manage_cutObjects', 'manage_copyObjects', 'manage_pasteObjects',
          'manage_renameForm', 'manage_renameObject',
          'manage_findFrame', 'manage_findForm', 'manage_findAdv',
          'manage_findResult', 'manage_findOpt')),
        ('Access contents information',
         ('objectIds', 'objectValues', 'objectItems','hasProperty',
          'propertyIds', 'propertyValues','propertyItems',''),
         ('Anonymous', 'Manager'),
         ),
        ('Undo changes',       ('manage_undo_transactions',)),
        ('Change permissions',
         ('manage_access','manage_changePermissions', 'manage_role',
          'manage_permission', 'manage_defined_roles',
          'manage_acquiredForm','manage_acquiredPermissions',
          'manage_permissionForm','manage_roleForm'
          )),
        ('Delete objects',     ('manage_delObjects',)),
        ('Manage properties',
         ('manage_addProperty', 'manage_editProperties',
          'manage_delProperties', 'manage_changeProperties',)),
    )

    manage_addObject__roles__=None

    def tpValues(self):
        """Returns a list of the folder's sub-folders, used by tree tag."""
        r=[]
        if hasattr(self.aq_base,'tree_ids'):
            for id in self.aq_base.tree_ids:
                if hasattr(self, id): r.append(getattr(self, id))
        else:
            for id in self._objects:
                o=getattr(self, id['id'])
                try:
                    if o.isPrincipiaFolderish: r.append(o)
#                   if subclass(o.__class__, Folder): r.append(o)
                except: pass

        return r

    def __getitem__(self, key):
        # Hm, getattr didn't work, maybe this is a put:
        if key[:19]=='manage_draftFolder-':
            id=key[19:]
            if hasattr(self, id): return getattr(self, id).manage_supervisor()
            raise KeyError, key
        try:
            if self.REQUEST['REQUEST_METHOD']=='PUT':
                return PUTer(self,key).__of__(self)
        except: pass
        raise KeyError, key

    def folderClass(self):
        return Folder
        return self.__class__

    test_url___allow_groups__=None
    def test_url_(self):
        """Test connection"""
        return 'PING'


    # The Following methods are short-term measures to get Paul off my back;)
    def manage_exportHack(self, id=None):
        """Exports a folder and its contents to /var/export.bbe
        This file can later be imported by using manage_importHack"""
        if id is None: o=self
        else: o=getattr(self.o)
        f=Globals.data_dir+'/export.bbe'
        o._p_jar.export_file(o,f)
        return f

    def manage_importHack(self, REQUEST=None):
        "Imports a previously exported object from /var/export.bbe"
        f=Globals.data_dir+'/export.bbe'
        o=self._p_jar.import_file(f)
        id=o.id
        if hasattr(id,'im_func'): id=id()
        self._setObject(id,o)
        return 'OK, I imported %s' % id



class PUTer(Acquisition.Explicit):
    """Class to support the HTTP PUT protocol."""

    def __init__(self, parent, id):
        self.id=id
        self.__parent__=parent
        self.__roles__ =parent.PUT__roles__

    def PUT(self, REQUEST, RESPONSE):
        """Adds a document, image or file to the folder when a PUT
        request is received."""
        name=self.id
        type=REQUEST.get_header('content-type', None)
        body=REQUEST.get('BODY', '')
        if type is None:
            type, enc=mimetypes.guess_type(name)
        if type is None:
            if content_types.find_binary(body) >= 0:
                raise 'Bad Request', 'Unknown content type'
            else: type=content_types.text_type(body)
        type=lower(type)
        if type in ('text/html', 'text/xml', 'text/plain'):
            self.__parent__.manage_addDTMLDocument(name, '', body)
        elif type[:6]=='image/':
            ob=Image(name, '', body, content_type=type)
            self.__parent__._setObject(name, ob)
        else:
            ob=File(name, '', body, content_type=type)
            self.__parent__._setObject(name, ob)
        RESPONSE.setStatus(201)
        RESPONSE.setBody('')
        return RESPONSE

    def __str__(self):
        return self.id






def subclass(c,super):
    if c is super: return 1
    try:
        for base in c.__bases__:
            if subclass(base,super): return 1
    except: pass
    return 0
