##############################################################################
# 
# Zope Public License (ZPL) Version 0.9.7
# ---------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
__doc__="""$Id: Scheduler.py,v 1.7 1998/12/04 20:15:31 jim Exp $"""

__version__='$Revision: 1.7 $'[11:-2]


from time import sleep, time, ctime
import Globals
import Main
import sys

try:
    import thread
    allocate_lock=thread.allocate_lock
except:
    thread=None
    class allocate_lock:
        def acquire(*args): pass
        def release(*args): pass



class Scheduler:
    def __init__(self):
        self.info={'lock': allocate_lock(),
                   'queue': [],
                   'checked': None,
                   }
        if thread: thread.start_new_thread(live,(self.info,))
        

    def schedule(self, event):
        """Register an event
        Where

        'register' returns the event ID of the event.
        
        lookup(eventID) -- Return the event with the given event ID
        
        cancel(eventID) -- Cancel the event with the given event ID
        """
        info=self.info
        info['lock'].acquire()
        info['queue'].append(event)
        info['lock'].release()
        if not thread:
            dispatch(info)



def Log(file='/dev/null'):
    try:    f=open(file, 'w')
    except: f=None
    if f is None:
        def write(m, file=f):
            pass
    else:
        def write(m, file=f):
            write=file.write
            write(ctime(time()))
            write('\t')
            write(str(m))
            write('\n')
            file.flush()
    return write



def live(info):
    log=Log()
    log('starting')
    while 1:
        log('start of loop')
        info['lock'].acquire()
        queue=info['queue'][:]
        del info['queue'][:]
        info['checked']=time()
        info['lock'].release()
        log('%s events' % len(queue))
        if queue:
            log(queue)
            for event in queue:
                try:
                    if event.threadsafe: event()
                    else:
                        Main.__bobo_before__()
                        t=get_transaction()
                        t.begin(event.name)
                        try:
                            event()
                            t.commit()
                            Main.__bobo_after__()
                        except:
                            log("%s: %s" % (sys.exc_type, sys.exc_traceback))
                            t.abort()
                            Main.__bobo_after__()
                except:
                    log("%s: %s" % (sys.exc_type, sys.exc_value))

        else:
            sleep(10)


def dispatch(info):
    # Handle events in non-threaded environment
    log=Log()
    log('dispatching events')

    queue=info['queue'][:]
    del info['queue'][:]
    info['checked']=time()

    if queue:
        log('Dispatching %s events' % len(queue))
        log(queue)
        for event in queue:
            # Since we have no threads, any events must
            # have been triggered by a user action, so
            # we live inside the current transaction...
            #
            # In the threaded Scheduler, an event that
            # blows up doesn't affect the user transaction
            # that effectively generated the event, so we
            # mimic that behavior here by just logging
            # events that go awry.
            try:    event()
            except: log("%s: %s" % (sys.exc_type, sys.exc_value))
        

Globals.Scheduler=Scheduler()
